import Node from '../node/Node';
import Element from '../element/Element';
/**
 * DocumentFragment.
 */
export default class DocumentFragment extends Node {
    nodeType: number;
    mode: string;
    children: Element[];
    /**
     * Query CSS selector to find matching nodes.
     *
     * @param selector CSS selector.
     * @returns Matching elements.
     */
    querySelectorAll(selector: string): Element[];
    /**
     * Query CSS Selector to find matching node.
     *
     * @param selector CSS selector.
     * @return Matching node.
     */
    querySelector(selector: string): Element;
    /**
     * Returns an element by ID.
     *
     * @param id ID.
     * @return Matching node.
     */
    getElementById(id: string): Element;
    /**
     * Returns an elements by tag name.
     *
     * @param tagName Tag name.
     * @returns Matching nodes.
     */
    getElementsByTagName(tagName: string): Element[];
    /**
     * Returns an elements by tag name.
     *
     * @param namespaceURI Namespace URI.
     * @param tagName Tag name.
     * @returns Matching nodes.
     */
    getElementsByTagNameNS(namespaceURI: string, tagName: string): Element[];
    /**
     * Returns an elements by class name.
     *
     * @param className Tag name.
     * @returns Matching nodes.
     */
    getElementsByClassName(className: string): Element[];
    /**
     * Clones a node.
     *
     * @override
     * @param [deep=false] "true" to clone deep.
     * @return Cloned node.
     */
    cloneNode(deep?: boolean): Node;
    /**
     * Append a child node to childNodes.
     *
     * @override
     * @param  node Node to append.
     * @return Appended node.
     */
    appendChild(node: Node): Node;
    /**
     * Remove Child element from childNodes array.
     *
     * @override
     * @param node Node to remove
     */
    removeChild(node: Node): void;
}
