"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var HTMLElement_1 = __importDefault(require("../html-element/HTMLElement"));
var TextNode_1 = __importDefault(require("../text-node/TextNode"));
var CommentNode_1 = __importDefault(require("../comment-node/CommentNode"));
var Node_1 = __importDefault(require("../node/Node"));
var TreeWalker_1 = __importDefault(require("../../../tree-walker/TreeWalker"));
var DocumentFragment_1 = __importDefault(require("../document-fragment/DocumentFragment"));
var XMLParser_1 = __importDefault(require("../../../xml-parser/XMLParser"));
var Event_1 = __importDefault(require("../../../event/Event"));
var DOMImplementation_1 = __importDefault(require("../../../dom-implementation/DOMImplementation"));
var HTMLElementTag_1 = __importDefault(require("../../../html-config/HTMLElementTag"));
var Attr_1 = __importDefault(require("../../../attribute/Attr"));
var NamespaceURI_1 = __importDefault(require("../../../html-config/NamespaceURI"));
var DocumentType_1 = __importDefault(require("../document-type/DocumentType"));
/**
 * Document.
 */
var Document = /** @class */ (function (_super) {
    __extends(Document, _super);
    /**
     * Creates an instance of Document.
     *
     * @param window Window instance.
     */
    function Document(window) {
        var _this = _super.call(this) || this;
        _this.nodeType = Node_1.default.DOCUMENT_NODE;
        _this._isConnected = true;
        _this._isFirstWrite = true;
        _this._isFirstWriteAfterOpen = false;
        _this.defaultView = window;
        _this.implementation = new DOMImplementation_1.default(window);
        var doctype = _this.implementation.createDocumentType('html', '', '');
        var documentElement = _this.createElement('html');
        var bodyElement = _this.createElement('body');
        var headElement = _this.createElement('head');
        _this.appendChild(doctype);
        _this.appendChild(documentElement);
        documentElement.appendChild(headElement);
        documentElement.appendChild(bodyElement);
        return _this;
    }
    Object.defineProperty(Document.prototype, "nodeName", {
        /**
         * Node name.
         *
         * @return Node name.
         */
        get: function () {
            return '#document';
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Document.prototype, "documentElement", {
        /**
         * Returns <html> element.
         *
         * @return Element.
         */
        get: function () {
            return this.querySelector('html');
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Document.prototype, "doctype", {
        /**
         * Returns document type element.
         *
         * @return Document type.
         */
        get: function () {
            for (var _i = 0, _a = this.childNodes; _i < _a.length; _i++) {
                var node = _a[_i];
                if (node instanceof DocumentType_1.default) {
                    return node;
                }
            }
            return null;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Document.prototype, "body", {
        /**
         * Returns <body> element.
         *
         * @return Element.
         */
        get: function () {
            return this.querySelector('body');
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Document.prototype, "head", {
        /**
         * Returns <head> element.
         *
         * @return Element.
         */
        get: function () {
            return this.querySelector('head');
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Replaces the document HTML with new HTML.
     *
     * @param html HTML.
     */
    Document.prototype.write = function (html) {
        var root = XMLParser_1.default.parse(this, html);
        if (this._isFirstWrite || this._isFirstWriteAfterOpen) {
            if (this._isFirstWrite) {
                if (!this._isFirstWriteAfterOpen) {
                    this.open();
                }
                this._isFirstWrite = false;
            }
            this._isFirstWriteAfterOpen = false;
            var documentElement = null;
            var documentTypeNode = null;
            for (var _i = 0, _a = root.childNodes; _i < _a.length; _i++) {
                var node = _a[_i];
                if (node['tagName'] === 'HTML') {
                    documentElement = node;
                }
                else if (node.nodeType === Node_1.default.DOCUMENT_TYPE_NODE) {
                    documentTypeNode = node;
                }
                if (documentElement && documentTypeNode) {
                    break;
                }
            }
            if (documentElement) {
                if (!this.documentElement) {
                    if (documentTypeNode) {
                        this.appendChild(documentTypeNode);
                    }
                    this.appendChild(documentElement);
                }
                else {
                    var rootBody = root.querySelector('body');
                    var body_1 = this.querySelector('body');
                    if (rootBody && body_1) {
                        for (var _b = 0, _c = rootBody.childNodes.slice(); _b < _c.length; _b++) {
                            var child = _c[_b];
                            body_1.appendChild(child);
                        }
                    }
                }
                var body = this.querySelector('body');
                if (body) {
                    for (var _d = 0, _e = root.childNodes.slice(); _d < _e.length; _d++) {
                        var child = _e[_d];
                        if (child['tagName'] !== 'HTML' && child.nodeType !== Node_1.default.DOCUMENT_TYPE_NODE) {
                            body.appendChild(child);
                        }
                    }
                }
            }
            else {
                var documentElement_1 = this.createElement('html');
                var bodyElement = this.createElement('body');
                var headElement = this.createElement('head');
                for (var _f = 0, _g = root.childNodes.slice(); _f < _g.length; _f++) {
                    var child = _g[_f];
                    bodyElement.appendChild(child);
                }
                documentElement_1.appendChild(headElement);
                documentElement_1.appendChild(bodyElement);
                this.appendChild(documentElement_1);
            }
        }
        else {
            var bodyNode = root.querySelector('body');
            for (var _h = 0, _j = (bodyNode || root).childNodes.slice(); _h < _j.length; _h++) {
                var child = _j[_h];
                this.body.appendChild(child);
            }
        }
    };
    /**
     * Opens the document.
     *
     * @returns Document.
     */
    Document.prototype.open = function () {
        this._isFirstWriteAfterOpen = true;
        for (var _i = 0, _a = Object.keys(this._listeners); _i < _a.length; _i++) {
            var eventType = _a[_i];
            var callbacks = this._listeners[eventType];
            if (callbacks) {
                for (var _b = 0, callbacks_1 = callbacks; _b < callbacks_1.length; _b++) {
                    var callback = callbacks_1[_b];
                    this.removeEventListener(eventType, callback);
                }
            }
        }
        for (var _c = 0, _d = this.childNodes.slice(); _c < _d.length; _c++) {
            var child = _d[_c];
            this.removeChild(child);
        }
        return this;
    };
    /**
     * Closes the document.
     */
    Document.prototype.close = function () { };
    /**
     * Creates an element.
     *
     * @param tagName Tag name.
     * @param [options] Options.
     * @return Element.
     */
    Document.prototype.createElement = function (tagName, options) {
        return this.createElementNS(NamespaceURI_1.default.html, tagName, options);
    };
    /**
     * Creates an element with the specified namespace URI and qualified name.
     *
     * @param tagName Tag name.
     * @param [options] Options.
     * @return Element.
     */
    Document.prototype.createElementNS = function (namespaceURI, qualifiedName, options) {
        var customElementClass = options && options.is
            ? this.defaultView.customElements.get(options.is)
            : this.defaultView.customElements.get(qualifiedName);
        var elementClass = customElementClass
            ? customElementClass
            : HTMLElementTag_1.default[qualifiedName] || HTMLElement_1.default;
        elementClass.ownerDocument = this;
        var element = new elementClass();
        element.tagName = qualifiedName.toUpperCase();
        element.ownerDocument = this;
        element._namespaceURI = namespaceURI;
        return element;
    };
    /**
     * Creates a text node.
     *
     * @param  data Text data.
     * @returns Text node.
     */
    Document.prototype.createTextNode = function (data) {
        TextNode_1.default.ownerDocument = this;
        var textNode = new TextNode_1.default();
        textNode.textContent = data;
        return textNode;
    };
    /**
     * Creates a comment node.
     *
     * @param  data Text data.
     * @returns Text node.
     */
    Document.prototype.createComment = function (data) {
        CommentNode_1.default.ownerDocument = this;
        var commentNode = new CommentNode_1.default();
        commentNode.textContent = data;
        return commentNode;
    };
    /**
     * Creates a document fragment.
     *
     * @returns Document fragment.
     */
    Document.prototype.createDocumentFragment = function () {
        DocumentFragment_1.default.ownerDocument = this;
        var documentFragment = new DocumentFragment_1.default();
        return documentFragment;
    };
    /**
     * Creates a Tree Walker.
     *
     * @param root Root.
     * @param [whatToShow] What to show.
     * @param [filter] Filter.
     */
    Document.prototype.createTreeWalker = function (root, whatToShow, filter) {
        if (whatToShow === void 0) { whatToShow = -1; }
        if (filter === void 0) { filter = null; }
        return new TreeWalker_1.default(root, whatToShow, filter);
    };
    /**
     * Creates an event.
     *
     * @legacy
     * @param _type Type.
     * @returns Event.
     */
    Document.prototype.createEvent = function (_type) {
        return new Event_1.default('init');
    };
    /**
     * Creates an Attr node.
     *
     * @param name Name.
     * @return Attribute.
     */
    Document.prototype.createAttribute = function (name) {
        var attribute = new Attr_1.default();
        attribute.name = name.toLowerCase();
        return attribute;
    };
    /**
     * Creates a namespaced Attr node.
     *
     * @param namespaceURI Namespace URI.
     * @param qualifiedName Qualified name.
     * @return Element.
     */
    Document.prototype.createAttributeNS = function (namespaceURI, qualifiedName) {
        var attribute = new Attr_1.default();
        attribute.namespaceURI = namespaceURI;
        attribute.name = qualifiedName;
        return attribute;
    };
    /**
     * Imports a node.
     *
     * @param node Node to import.
     * @param Imported node.
     */
    Document.prototype.importNode = function (node) {
        if (!(node instanceof Node_1.default)) {
            throw new Error('Parameter 1 was not of type Node.');
        }
        var clone = node.cloneNode(true);
        clone.ownerDocument = this;
        return clone;
    };
    return Document;
}(DocumentFragment_1.default));
exports.default = Document;
//# sourceMappingURL=Document.js.map