"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var Node_1 = __importDefault(require("../node/Node"));
var TextNode_1 = __importDefault(require("../text-node/TextNode"));
var ShadowRoot_1 = __importDefault(require("../shadow-root/ShadowRoot"));
var Attr_1 = __importDefault(require("../../../attribute/Attr"));
var DOMRect_1 = __importDefault(require("./DOMRect"));
var Range_1 = __importDefault(require("./Range"));
var ClassList_1 = __importDefault(require("./ClassList"));
var QuerySelector_1 = __importDefault(require("../../../query-selector/QuerySelector"));
var MutationRecord_1 = __importDefault(require("../../../mutation-observer/MutationRecord"));
var MutationType_1 = __importDefault(require("../../../mutation-observer/MutationType"));
var NamespaceURI_1 = __importDefault(require("../../../html-config/NamespaceURI"));
var XMLParser_1 = __importDefault(require("../../../xml-parser/XMLParser"));
var XMLSerializer_1 = __importDefault(require("../../../xml-serializer/XMLSerializer"));
/**
 * Element.
 */
var Element = /** @class */ (function (_super) {
    __extends(Element, _super);
    function Element() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.tagName = null;
        _this.nodeType = Node_1.default.ELEMENT_NODE;
        _this.shadowRoot = null;
        _this.classList = new ClassList_1.default(_this);
        _this.scrollTop = 0;
        _this.scrollLeft = 0;
        _this.children = [];
        _this._attributes = {};
        _this._namespaceURI = null;
        return _this;
    }
    Object.defineProperty(Element, "observedAttributes", {
        /**
         * Returns a list of observed attributes.
         *
         * @return Observered attributes.
         */
        get: function () {
            return undefined;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Element.prototype, "id", {
        /**
         * Returns ID.
         *
         * @return ID.
         */
        get: function () {
            return this.getAttribute('id') || '';
        },
        /**
         * Sets ID.
         *
         * @param id ID.
         * @return HTML.
         */
        set: function (id) {
            this.setAttribute('id', id);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Element.prototype, "className", {
        /**
         * Returns class name.
         *
         * @return Class name.
         */
        get: function () {
            return this.getAttribute('class') || '';
        },
        /**
         * Sets class name.
         *
         * @param className Class name.
         * @return Class name.
         */
        set: function (className) {
            this.setAttribute('class', className);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Element.prototype, "nodeName", {
        /**
         * Node name.
         *
         * @return Node name.
         */
        get: function () {
            return this.tagName;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Element.prototype, "namespaceURI", {
        /**
         * Returns the namespace URI.
         *
         * @return Namespace URI.
         */
        get: function () {
            return this._namespaceURI;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Element.prototype, "textContent", {
        /**
         * Get text value of children.
         *
         * @return Text content.
         */
        get: function () {
            var result = '';
            for (var _i = 0, _a = this.childNodes; _i < _a.length; _i++) {
                var childNode = _a[_i];
                if (childNode instanceof Element || childNode instanceof TextNode_1.default) {
                    result += childNode.textContent;
                }
            }
            return result;
        },
        /**
         * Sets text content.
         *
         * @param textContent Text content.
         */
        set: function (textContent) {
            for (var _i = 0, _a = this.childNodes.slice(); _i < _a.length; _i++) {
                var child = _a[_i];
                this.removeChild(child);
            }
            this.appendChild(this.ownerDocument.createTextNode(textContent));
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Element.prototype, "innerHTML", {
        /**
         * Returns inner HTML.
         *
         * @return HTML.
         */
        get: function () {
            var xmlSerializer = new XMLSerializer_1.default();
            var xml = '';
            for (var _i = 0, _a = this.childNodes; _i < _a.length; _i++) {
                var node = _a[_i];
                xml += xmlSerializer.serializeToString(node);
            }
            return xml;
        },
        /**
         * Sets inner HTML.
         *
         * @param html HTML.
         */
        set: function (html) {
            for (var _i = 0, _a = this.childNodes.slice(); _i < _a.length; _i++) {
                var child = _a[_i];
                this.removeChild(child);
            }
            for (var _b = 0, _c = XMLParser_1.default.parse(this.ownerDocument, html).childNodes.slice(); _b < _c.length; _b++) {
                var node = _c[_b];
                this.appendChild(node);
            }
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Element.prototype, "outerHTML", {
        /**
         * Returns outer HTML.
         *
         * @return HTML.
         */
        get: function () {
            return new XMLSerializer_1.default().serializeToString(this);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Element.prototype, "attributes", {
        /**
         * Returns attributes.
         *
         * @returns Attributes.
         */
        get: function () {
            var attributes = Object.values(this._attributes);
            return Object.assign({}, this._attributes, attributes, {
                length: attributes.length
            });
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Clones a node.
     *
     * @override
     * @param [deep=false] "true" to clone deep.
     * @return Cloned node.
     */
    Element.prototype.cloneNode = function (deep) {
        if (deep === void 0) { deep = false; }
        var clone = _super.prototype.cloneNode.call(this, deep);
        for (var _i = 0, _a = Object.keys(this._attributes); _i < _a.length; _i++) {
            var key = _a[_i];
            clone._attributes[key] = Object.assign(new Attr_1.default(), this._attributes[key]);
        }
        if (deep) {
            clone.children = (clone.childNodes.filter(function (node) { return node.nodeType === Node_1.default.ELEMENT_NODE; }));
        }
        clone.tagName = this.tagName;
        clone.scrollLeft = this.scrollLeft;
        clone.scrollTop = this.scrollTop;
        clone._namespaceURI = this._namespaceURI;
        return clone;
    };
    /**
     * Append a child node to childNodes.
     *
     * @override
     * @param  node Node to append.
     * @return Appended node.
     */
    Element.prototype.appendChild = function (node) {
        if (node !== this && node instanceof Element) {
            this.children.push(node);
        }
        return _super.prototype.appendChild.call(this, node);
    };
    /**
     * Remove Child element from childNodes array.
     *
     * @override
     * @param node Node to remove
     */
    Element.prototype.removeChild = function (node) {
        if (node instanceof Element) {
            var index = this.children.indexOf(node);
            if (index !== -1) {
                this.children.splice(index, 1);
            }
        }
        _super.prototype.removeChild.call(this, node);
    };
    /**
     * Sets an attribute.
     *
     * @param name Name.
     * @param value Value.
     */
    Element.prototype.setAttribute = function (name, value) {
        var attribute = this.ownerDocument.createAttributeNS(null, name);
        attribute.value = String(value);
        this.setAttributeNode(attribute);
    };
    /**
     * Sets a namespace attribute.
     *
     * @param namespaceURI Namespace URI.
     * @param name Name.
     * @param value Value.
     */
    Element.prototype.setAttributeNS = function (namespaceURI, name, value) {
        var attribute = this.ownerDocument.createAttributeNS(namespaceURI, name);
        attribute.value = String(value);
        this.setAttributeNode(attribute);
    };
    /**
     * Returns attribute value.
     *
     * @param name Name.
     */
    Element.prototype.getAttribute = function (name) {
        var attribute = this.getAttributeNode(name);
        if (attribute) {
            return attribute.value;
        }
        return null;
    };
    /**
     * Returns namespace attribute value.
     *
     * @param namespace Namespace URI.
     * @param localName Local name.
     */
    Element.prototype.getAttributeNS = function (namespace, localName) {
        for (var _i = 0, _a = Object.keys(this._attributes); _i < _a.length; _i++) {
            var name = _a[_i];
            var attribute = this._attributes[name];
            if (attribute.namespaceURI === namespace && attribute.localName === localName) {
                return attribute.value;
            }
        }
        return null;
    };
    /**
     * Returns a boolean value indicating whether the specified element has the attribute or not.
     *
     * @param name Attribute name.
     * @returns True if attribute exists, false otherwise.
     */
    Element.prototype.hasAttribute = function (name) {
        return !!this.getAttributeNode(name);
    };
    /**
     * Returns a boolean value indicating whether the specified element has the namespace attribute or not.
     *
     * @param namespace Namespace URI.
     * @param localName Local name.
     * @returns True if attribute exists, false otherwise.
     */
    Element.prototype.hasAttributeNS = function (namespace, localName) {
        for (var _i = 0, _a = Object.keys(this._attributes); _i < _a.length; _i++) {
            var name = _a[_i];
            var attribute = this._attributes[name];
            if (attribute.namespaceURI === namespace && attribute.localName === localName) {
                return true;
            }
        }
        return false;
    };
    /**
     * Returns "true" if the element has attributes.
     *
     * @return "true" if the element has attributes.
     */
    Element.prototype.hasAttributes = function () {
        return Object.keys(this._attributes).length > 0;
    };
    /**
     * Removes an attribute.
     *
     * @param name Name.
     */
    Element.prototype.removeAttribute = function (name) {
        var attribute = this._attributes[this._getAttributeName(name)];
        if (attribute) {
            this.removeAttributeNode(attribute);
        }
    };
    /**
     * Removes a namespace attribute.
     *
     * @param namespace Namespace URI.
     * @param localName Local name.
     */
    Element.prototype.removeAttributeNS = function (namespace, localName) {
        for (var _i = 0, _a = Object.keys(this._attributes); _i < _a.length; _i++) {
            var name = _a[_i];
            var attribute = this._attributes[name];
            if (attribute.namespaceURI === namespace && attribute.localName === localName) {
                this.removeAttribute(attribute.name);
            }
        }
    };
    /**
     * Attaches a shadow root.
     *
     * @param _shadowRootInit Shadow root init.
     * @returns Shadow root.
     */
    Element.prototype.attachShadow = function (_shadowRootInit) {
        if (this.shadowRoot) {
            throw new Error('Shadow root has already been attached.');
        }
        this.shadowRoot = new ShadowRoot_1.default();
        this.shadowRoot.ownerDocument = this.ownerDocument;
        this.shadowRoot.isConnected = this.isConnected;
        return this.shadowRoot;
    };
    /**
     * Scrolls to a particular set of coordinates in the document.
     *
     * @note This method has not been implemented. It is just here for compatibility.
     */
    Element.prototype.scrollTo = function () { };
    /**
     * Converts to string.
     *
     * @return String.
     */
    Element.prototype.toString = function () {
        return this.outerHTML;
    };
    /**
     * Returns the size of an element and its position relative to the viewport.
     *
     * @returns DOM rect.
     */
    Element.prototype.getBoundingClientRect = function () {
        return new DOMRect_1.default();
    };
    /**
     * Returns a range.
     *
     * @returns Range.
     */
    Element.prototype.createTextRange = function () {
        return new Range_1.default();
    };
    /**
     * Query CSS selector to find matching nodes.
     *
     * @param selector CSS selector.
     * @returns Matching elements.
     */
    Element.prototype.querySelectorAll = function (selector) {
        return QuerySelector_1.default.querySelectorAll(this, selector);
    };
    /**
     * Query CSS Selector to find matching node.
     *
     * @param selector CSS selector.
     * @return Matching node.
     */
    Element.prototype.querySelector = function (selector) {
        return QuerySelector_1.default.querySelector(this, selector);
    };
    /**
     * Returns an elements by tag name.
     *
     * @param tagName Tag name.
     * @returns Matching nodes.
     */
    Element.prototype.getElementsByTagName = function (tagName) {
        return this.querySelectorAll(tagName);
    };
    /**
     * Returns an elements by tag name.
     *
     * @param namespaceURI Namespace URI.
     * @param tagName Tag name.
     * @returns Matching nodes.
     */
    Element.prototype.getElementsByTagNameNS = function (namespaceURI, tagName) {
        return this.querySelectorAll(tagName).filter(function (element) { return element._namespaceURI === namespaceURI; });
    };
    /**
     * Returns an elements by class name.
     *
     * @param className Tag name.
     * @returns Matching nodes.
     */
    Element.prototype.getElementsByClassName = function (className) {
        return this.querySelectorAll('.' + className.split(' ').join('.'));
    };
    /**
     * The setAttributeNode() method adds a new Attr node to the specified element.
     *
     * @param attribute Attribute.
     * @returns Replaced attribute.
     */
    Element.prototype.setAttributeNode = function (attribute) {
        var name = this._getAttributeName(attribute.name);
        var replacedAttribute = this._attributes[name];
        var oldValue = replacedAttribute ? replacedAttribute.value : null;
        attribute.name = name;
        this._attributes[name] = attribute;
        // Styles
        if (name === 'style' && this['style']) {
            for (var _i = 0, _a = attribute.value.split(';'); _i < _a.length; _i++) {
                var part = _a[_i];
                var _b = part.split(':'), key = _b[0], value = _b[1];
                if (key && value) {
                    this['style'][this._kebabToCamelCase(key.trim())] = value.trim();
                }
            }
        }
        if (this.attributeChangedCallback &&
            this.constructor.observedAttributes &&
            this.constructor.observedAttributes.includes(name)) {
            this.attributeChangedCallback(name, oldValue, attribute.value);
        }
        // MutationObserver
        if (this._observers.length > 0) {
            for (var _c = 0, _d = this._observers; _c < _d.length; _c++) {
                var observer = _d[_c];
                if (observer.options.attributes &&
                    (!observer.options.attributeFilter || observer.options.attributeFilter.includes(name))) {
                    var record = new MutationRecord_1.default();
                    record.type = MutationType_1.default.attributes;
                    record.attributeName = name;
                    record.oldValue = observer.options.attributeOldValue ? oldValue : null;
                    observer.callback([record]);
                }
            }
        }
        return replacedAttribute || null;
    };
    /**
     * Returns an Attr node.
     *
     * @param name Name.
     * @returns Replaced attribute.
     */
    Element.prototype.getAttributeNode = function (name) {
        return this._attributes[this._getAttributeName(name)] || null;
    };
    /**
     * Removes an Attr node.
     *
     * @param attribute Attribute.
     */
    Element.prototype.removeAttributeNode = function (attribute) {
        delete this._attributes[attribute.name];
        if (this.attributeChangedCallback &&
            this.constructor.observedAttributes &&
            this.constructor.observedAttributes.includes(attribute.name)) {
            this.attributeChangedCallback(attribute.name, attribute.value, null);
        }
        // MutationObserver
        if (this._observers.length > 0) {
            for (var _i = 0, _a = this._observers; _i < _a.length; _i++) {
                var observer = _a[_i];
                if (observer.options.attributes &&
                    (!observer.options.attributeFilter ||
                        observer.options.attributeFilter.includes(attribute.name))) {
                    var record = new MutationRecord_1.default();
                    record.type = MutationType_1.default.attributes;
                    record.attributeName = attribute.name;
                    record.oldValue = observer.options.attributeOldValue ? attribute.value : null;
                    observer.callback([record]);
                }
            }
        }
    };
    /**
     * Returns attribute name.
     *
     * @param name Name.
     * @returns Attribute name based on namespace.
     */
    Element.prototype._getAttributeName = function (name) {
        if (this._namespaceURI === NamespaceURI_1.default.svg) {
            return name;
        }
        return name.toLowerCase();
    };
    /**
     * Kebab case to words.
     *
     * @param string String to convert.
     * @returns Text as kebab case.
     */
    Element.prototype._kebabToCamelCase = function (string) {
        string = string.split('-');
        for (var i = 0, max = string.length; i < max; i++) {
            var firstWord = i > 0 ? string[i].charAt(0).toUpperCase() : string[i].charAt(0);
            string[i] = firstWord + string[i].slice(1);
        }
        return string.join('');
    };
    return Element;
}(Node_1.default));
exports.default = Element;
//# sourceMappingURL=Element.js.map