"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var Element_1 = __importDefault(require("../nodes/basic/element/Element"));
var SelfClosingHTMLElements_1 = __importDefault(require("../html-config/SelfClosingHTMLElements"));
var UnclosedHTMLElements_1 = __importDefault(require("../html-config/UnclosedHTMLElements"));
var CommentNode_1 = __importDefault(require("../nodes/basic/comment-node/CommentNode"));
var DocumentType_1 = __importDefault(require("../nodes/basic/document-type/DocumentType"));
var he_1 = require("he");
var DocumentFragment_1 = __importDefault(require("../nodes/basic/document-fragment/DocumentFragment"));
/**
 * Utility for converting an element to string.
 *
 * @class QuerySelector
 */
var XMLSerializer = /** @class */ (function () {
    function XMLSerializer() {
    }
    /**
     * Renders an element as HTML.
     *
     * @param element Element to render.
     * @return Result.
     */
    XMLSerializer.prototype.serializeToString = function (root) {
        if (root instanceof Element_1.default) {
            var tagName = root.tagName.toLowerCase();
            if (UnclosedHTMLElements_1.default.includes(tagName)) {
                return "<" + tagName + this._getAttributes(root) + ">";
            }
            else if (SelfClosingHTMLElements_1.default.includes(tagName)) {
                return "<" + tagName + this._getAttributes(root) + "/>";
            }
            var xml = '';
            for (var _i = 0, _a = root.childNodes; _i < _a.length; _i++) {
                var node = _a[_i];
                xml += this.serializeToString(node);
            }
            return "<" + tagName + this._getAttributes(root) + ">" + xml + "</" + tagName + ">";
        }
        else if (root instanceof DocumentFragment_1.default) {
            var xml = '';
            for (var _b = 0, _c = root.childNodes; _b < _c.length; _b++) {
                var node = _c[_b];
                xml += this.serializeToString(node);
            }
            return xml;
        }
        else if (root instanceof CommentNode_1.default) {
            return "<!--" + root._textContent + "-->";
        }
        else if (root instanceof DocumentType_1.default) {
            var identifier = root.publicId ? ' PUBLIC' : root.systemId ? ' SYSTEM' : '';
            var publicId = root.publicId ? " \"" + root.publicId + "\"" : '';
            var systemId = root.systemId ? " \"" + root.systemId + "\"" : '';
            return "<!DOCTYPE " + root.name + identifier + publicId + systemId + ">";
        }
        else if (root['_textContent']) {
            return root['_textContent'];
        }
        return '';
    };
    /**
     * Returns attributes as a string.
     *
     * @param element Element.
     * @return Attributes.
     */
    XMLSerializer.prototype._getAttributes = function (element) {
        var attributes = [];
        for (var _i = 0, _a = Object.values(element._attributes); _i < _a.length; _i++) {
            var attribute = _a[_i];
            if (attribute.value !== null) {
                attributes.push(attribute.name + '="' + he_1.encode(attribute.value) + '"');
            }
        }
        return attributes.length > 0 ? ' ' + attributes.join(' ') : '';
    };
    return XMLSerializer;
}());
exports.default = XMLSerializer;
//# sourceMappingURL=XMLSerializer.js.map